@@ -28,6 +28,7 @@
 
 #ifdef	_MSWINDOWS_
 #include <process.h>
+#include <winreg.h>
 #else
 #include <sys/wait.h>
 #include <sys/ioctl.h>
@@ -54,8 +55,17 @@
 #define	textdomain(s)
 #endif
 
+#ifdef	HAVE_SYSLOG_H
+#include <syslog.h>
+#endif
+
 using namespace UCOMMON_NAMESPACE;
 
+static shell::loglevel_t errlevel = shell::WARN;
+static shell::logmode_t errmode = shell::NONE;
+static const char *errname = NULL;
+static shell::logproc_t errproc = (shell::logproc_t)NULL;
+
 OrderedIndex shell::Option::index;
 const char *shell::domain = NULL;
 
@@ -362,6 +372,7 @@ OrderedObject(&index)
 	long_option = longopt;
 	uses_option = value;
 	help_string = help;
+	trigger_option = false;
 }
 
 shell::Option::~Option()
@@ -414,6 +425,36 @@ const char *shell::numericopt::assign(const char *value)
 	return NULL;
 }
 
+shell::counteropt::counteropt(char short_option, const char *long_option, const char *help_string, const char *type, long def_value) :
+shell::Option(short_option, long_option, type, help_string)
+{
+	used = false;
+	number = def_value;
+	trigger_option = true;
+}
+
+const char *shell::counteropt::assign(const char *value)
+{
+	char *endptr = NULL;
+
+	// trigger option mode received...
+	if(value == NULL) {
+		++number;
+		used = true;
+		return NULL;
+	}
+
+	if(used)
+		return errmsg(shell::OPTION_USED);
+
+	used = true;
+	number = strtol(value, &endptr, 0);
+	if(!endptr || *endptr != 0)
+		return errmsg(shell::BAD_VALUE);
+
+	return NULL;
+}
+
 shell::stringopt::stringopt(char short_option, const char *long_option, const char *help_string, const char *type, const char *def_value) :
 shell::Option(short_option, long_option, type, help_string)
 {
@@ -583,7 +624,11 @@ void shell::help(void)
 			op.next();
 			continue;
 		}
-		if(op->short_option && op->long_option) {
+		if(op->short_option && op->long_option && op->uses_option && !op->trigger_option) {
+			printf("  -%c .., ", op->short_option);
+			hp = 9;
+		}
+		else if(op->short_option && op->long_option) {
 			printf("  -%c, ", op->short_option);
 			hp = 6;
 		}
@@ -806,6 +851,9 @@ char **shell::getargv(char **argv)
 
 			value = NULL;
 
+			if(op->trigger_option) 
+				goto trigger;
+
 			if(op->uses_option && arg[1] == 0)
 				value = argv[argp++];
 			else if(op->uses_option)
@@ -813,10 +861,11 @@ char **shell::getargv(char **argv)
 
 			if(op->uses_option && value == NULL)
 				errexit(1, "*** %s: -%c: %s\n", _argv0, op->short_option, errmsg(shell::NOARGUMENT));
+trigger:
 			err = op->assign(value);
 			if(err)
 				errexit(1, "*** %s: -%c: %s\n", _argv0, op->short_option, shell::text(err));
-			if(op->uses_option)
+			if(value)
 				break;
 		}
 	}
@@ -898,11 +947,26 @@ skip:
 void shell::errexit(int exitcode, const char *format, ...)
 {
 	va_list args;
+	loglevel_t level = ERR;
 
 	va_start(args, format);
 	vfprintf(stderr, format, args);
 	fflush(stderr);
+
+	if(exitcode)
+		level = FAIL;
+
+#ifdef	HAVE_SYSLOG_H
+	if(errname && errmode != NONE && level <= errlevel) {
+		if(exitcode)
+			vsyslog(LOG_CRIT, format, args);
+		else
+			vsyslog(LOG_ERR, format, args);
+	}
+#endif
+
 	va_end(args);
+	
 	if(exitcode)
 		::exit(exitcode);
 }
@@ -1071,11 +1135,33 @@ tail:
 const char *shell::getenv(const char *id, const char *value)
 {
 	char buf[512];
+	char path[255];
+	const char *keyid = NULL;
 
-	if(!GetEnvironmentVariable(id, buf, sizeof(buf)))
-		return value;
+	if(GetEnvironmentVariable(id, buf, sizeof(buf)))
+		return dup(buf);
 
-	return dup(buf);
+	if(errname)
+		keyid = errname;
+	else if(domain)
+		keyid = domain;
+	else if(_argv0)
+		keyid = _argv0;
+	
+	if(keyid) {
+		snprintf(path, sizeof(path), "Default Environment\\%s", keyid);
+		HKEY key;
+		if(RegOpenKey(HKEY_CLASSES_ROOT, path, &key) == ERROR_SUCCESS) {
+			LONG dlen = sizeof(buf);
+			buf[0] = 0;
+			RegQueryValueA(key, id, (LPTSTR)buf, &dlen);
+			RegCloseKey(key);
+			if(buf[0])
+				return dup(buf);
+		}	
+	}
+	
+	return value;		
 }
 	
 int shell::system(const char *cmd, const char **envp)
@@ -1584,12 +1670,26 @@ const char *shell::text(const char *msg)
 
 void shell::bind(const char *name)
 {
-	if(!domain) {
-		setlocale(LC_ALL, "");
-		bindtextdomain("ucommon", UCOMMON_LOCALE);
+#ifdef	_MSWINDOWS_
+	const char *locale = "C:\\Program Files\\Common Files\\locale";
+
+	if(fsys::isdir("locale")) {
+		char prefix[256];
+		fsys::getPrefix(prefix, sizeof(prefix));
+		String::add(prefix, sizeof(prefix), "/locale");	
+		locale = strdup(prefix);
 	}
 
-	bindtextdomain(name, UCOMMON_LOCALE);
+#else
+	const char *locale = UCOMMON_LOCALE;
+#endif
+
+	if(!domain) {
+		setlocale(LC_ALL, "");
+		bindtextdomain("ucommon", locale);
+	}
+
+	bindtextdomain(name, locale);
 	textdomain(name);
 	domain = name;
 }
@@ -1602,3 +1702,176 @@ void shell::rebind(const char *name)
 		textdomain(domain);
 }
 
+#ifdef	_MSWINDOWS_
+void shell::priority(int level)
+{
+}
+#else
+void shell::priority(int level)
+{
+#if _POSIX_PRIORITY_SCHEDULING > 0
+    int policy = SCHED_OTHER;
+
+    if(level > 0)
+        policy = SCHED_RR;
+
+    struct sched_param sparam;
+    int min = sched_get_priority_min(policy);
+    int max = sched_get_priority_max(policy);
+    int pri = (int)level;
+
+    if(min == max)
+        pri = min;
+    else
+        pri += min;
+    if(pri > max)
+        pri = max;
+
+    setpriority(PRIO_PROCESS, 0, -level);
+    memset(&sparam, 0, sizeof(sparam));
+    sparam.sched_priority = pri;
+    sched_setscheduler(0, policy, &sparam);
+#else
+    nice(-level);
+#endif
+}
+#endif
+
+#ifdef	HAVE_SYSLOG_H
+
+void shell::log(const char *name, loglevel_t level, logmode_t mode, logproc_t handler)
+{
+	errlevel = level;
+	errmode = mode;
+	errname = name;
+
+	if(handler != (logproc_t)NULL)
+		errproc = handler;
+
+	switch(mode) {
+	case NONE:
+		closelog();
+		return;
+	case CONSOLE_LOG:
+		::openlog(name, LOG_CONS, LOG_DAEMON);
+		return;
+	case USER_LOG:
+		::openlog(name, 0, LOG_USER);
+		return;
+	case SYSTEM_LOG:
+		::openlog(name, LOG_CONS, LOG_DAEMON);
+		return;
+	}
+}
+
+void shell::log(loglevel_t loglevel, const char *fmt, ...)
+{
+    assert(fmt != NULL && *fmt != 0);
+
+    char buf[256];
+    va_list args;
+	int level= LOG_ERR;
+
+	if(!errname || errmode == NONE || loglevel > errlevel)
+		return;
+
+    va_start(args, fmt);
+    vsnprintf(buf, sizeof(buf), fmt, args);
+    va_end(args);
+
+	if(errproc != (logproc_t)NULL) {
+		if((*errproc)(loglevel, buf))
+			return;
+	}
+
+	if(loglevel >= DEBUG0) {
+        if(getppid() > 1) {
+            if(fmt[strlen(fmt) - 1] == '\n')
+                fprintf(stderr, "%s: %s", errname, buf);
+            else
+                fprintf(stderr, "%s: %s\n", errname, buf);
+        }
+		return;
+	}
+
+	switch(loglevel) {
+	case INFO:
+		level = LOG_INFO;
+		break;
+	case NOTIFY:
+		level = LOG_NOTICE;
+		break;
+	case WARN:
+		level = LOG_WARNING;
+		break;
+	case ERR:
+		level = LOG_ERR;
+		break;
+	case FAIL:
+		level = LOG_CRIT;
+		break;
+	default:
+		level = LOG_ERR;
+	}
+
+	if(getppid() > 1) {
+		if(fmt[strlen(fmt) - 1] == '\n')
+			fprintf(stderr, "%s: %s", errname, buf);
+		else
+			fprintf(stderr, "%s: %s\n", errname, buf);
+	}
+	::syslog(level, "%s", buf);
+
+	if(level == LOG_CRIT)
+		cpr_runtime_error(buf);
+}
+		
+#else
+
+void shell::log(const char *name, loglevel_t level, logmode_t mode, logproc_t handler)
+{
+	errlevel = level;
+	errmode = mode;
+	errname = name;
+
+	if(handler != (logproc_t)NULL)
+		errproc = handler;
+}
+
+void shell::log(loglevel_t loglevel, const char *fmt, ...)
+{
+    assert(fmt != NULL && *fmt != 0);
+
+    char buf[256];
+    va_list args;
+
+	if(!errname || errmode == NONE || loglevel > errlevel)
+		return;
+
+    va_start(args, fmt);
+    vsnprintf(buf, sizeof(buf), fmt, args);
+    va_end(args);
+
+	if(errproc != (logproc_t)NULL) {
+		if((*errproc)(loglevel, buf))
+			return;
+	}
+
+	if(loglevel >= DEBUG0) {
+		if(fmt[strlen(fmt) - 1] == '\n')
+			fprintf(stderr, "%s: %s", errname, buf);
+		else
+			fprintf(stderr, "%s: %s\n", errname, buf);
+		return;
+	}
+
+	if(fmt[strlen(fmt) - 1] == '\n')
+		fprintf(stderr, "%s: %s", errname, buf);
+	else
+		fprintf(stderr, "%s: %s\n", errname, buf);
+
+	if(loglevel == FAIL)
+		cpr_runtime_error(buf);
+}
+		
+#endif
