@@ -87,6 +87,21 @@ public:
 	 */
 	typedef enum {NOARGS = 0, NOARGUMENT, INVARGUMENT, BADOPTION, OPTION_USED, BAD_VALUE} errmsg_t;
 
+	/**
+	 * Type of error logging we are using.
+	 */
+	typedef enum {NONE = 0, CONSOLE_LOG, USER_LOG, SYSTEM_LOG} logmode_t;
+
+	/**
+	 * Level of error logging.
+	 */
+	typedef enum {FAIL = 0, ERR, WARN, NOTIFY, INFO, DEBUG0} loglevel_t;
+
+	/**
+	 * Log process handler.
+	 */
+	typedef bool (*logproc_t)(loglevel_t level, const char *text);
+
 #ifdef	_MSWINDOWS_
 	typedef	HANDLE pid_t;
 #else
@@ -287,6 +302,7 @@ public:
 		const char *long_option;
 		const char *uses_option;
 		const char *help_string;
+		bool trigger_option;
 
 		/**
 		 * Construct a shell parser option.
@@ -348,6 +364,9 @@ public:
 
 		inline unsigned operator*()
 			{return counter;};
+
+		inline void set(unsigned value = 1)
+			{counter = value;};
 	};
 
 	/**
@@ -369,6 +388,9 @@ public:
 	public:
 		stringopt(char short_option, const char *long_option = NULL, const char *help = NULL, const char *type = "text", const char *def_text = NULL);
 
+		inline void set(const char *string)
+			{text = string;};
+
 		inline operator bool()
 			{return used;};
 
@@ -403,6 +425,9 @@ public:
 	public:
 		charopt(char short_option, const char *long_option = NULL, const char *help = NULL, const char *type = "char", char default_code = ' ');
 
+		inline void set(char value)
+			{code = value;};
+
 		inline operator bool()
 			{return used;};
 
@@ -435,6 +460,46 @@ public:
 	public:
 		numericopt(char short_option, const char *long_option = NULL, const char *help = NULL, const char *type = "numeric", long def_value = 0);
 
+		inline void set(long value)
+			{number = value;};
+
+		inline operator bool()
+			{return used;};
+
+		inline bool operator!()
+			{return !used;};
+
+		inline operator long()
+			{return number;};
+
+		inline long operator*()
+			{return number;};
+	};
+
+	/**
+	 * Counter option for shell parsing.  This offers a quick-use class
+	 * to parse a shell flag, along with a numberic value that may be
+	 * saved and a use counter, as multiple invokations is an error.  Unlike
+	 * numeric options, the short mode flag is a trigger option, and each
+	 * use of the short flag is considered a counter increment.
+	 * @author David Sugar <dyfet@gnutelephony.org>
+	 */
+	class __EXPORT counteropt : public Option
+	{
+	private:
+		bool used;
+
+	protected:
+		long number;
+
+		virtual const char *assign(const char *value);
+
+	public:
+		counteropt(char short_option, const char *long_option = NULL, const char *help = NULL, const char *type = "numeric", long def_value = 0);
+
+		inline void set(long value)
+			{number = value;};
+
 		inline operator bool()
 			{return used;};
 
@@ -583,6 +648,22 @@ public:
 	 */
 	static void errexit(int exitcode, const char *format = NULL, ...) __PRINTF(2, 3);
 
+	/**
+	 * Print error message at specific error level.
+	 * @param level of error condition.
+	 * @param format string to use.
+	 */
+	static void log(loglevel_t level, const char *format, ...) __PRINTF(2, 3);
+
+	/**
+	 * Set logging level and state.
+	 * @param name of logging entity.
+	 * @param level of error conditions to log.
+	 * @param mode of logging.
+	 * @param handler for log messages.
+	 */
+	static void log(const char *name, loglevel_t level = ERR, logmode_t = USER_LOG, logproc_t handler = (logproc_t)NULL);
+
 	/**
 	 * Print to standard output.
 	 * @param format string to use.
@@ -673,6 +754,16 @@ public:
 	 */
 	static shell::pipe_t spawn(const char *path, char **argv, pmode_t mode, size_t size = 512, char **env = NULL);
 
+	/**
+	 * Set priority level and enable priority scheduler.  This activates the
+	 * realtime priority scheduler when a priority > 0 is requested for the
+	 * process, assuming scheduler support exists and the process is
+	 * sufficiently privileged.  Negative priorities are essentially the
+	 * same as nice.
+	 * @param priority level for process.
+	 */
+	static void priority(int level = 1);
+
 	/**
 	 * Create a detached process.  This creates a new child process that
 	 * is completely detached from the current process.
