@@ -262,6 +262,18 @@ public:
 	 */
 	string(const StringFormat& format);
 
+	/**
+	 * Create a string from a long integer.
+	 * @param value to convert to string.
+	 */
+	string(long value);
+
+	/**
+	 * Create a string from a floating point.
+	 * @param value to convert to string.
+	 */
+	string(double value);
+
 	/**
 	 * Create an empty string with a buffer pre-allocated to a specified size.
 	 * @param size of buffer to allocate.
@@ -277,12 +289,14 @@ public:
 
 	/**
 	 * Create a string by printf-like formating into a pre-allocated space
-	 * of a specified size.
+	 * of a specified size.  A typical use might be in a concat function
+	 * like String x = (String)something + (String){10, "%ud", var}.
 	 * @param size of buffer to allocate.
 	 * @param format control for string.
 	 */
 	string(strsize_t size, const char *format, ...) __PRINTF(3, 4);
 
+
 	/**
 	 * Create a string from null terminated text.
 	 * @param text to use for string.
